'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var child_process = require('child_process');
var getNpmCliPath = _interopDefault(require('npm-cli-path'));
var resolve = require('@pagedip/util-resolve');
var resolve__default = _interopDefault(resolve);
var fs = _interopDefault(require('mz/fs'));
var child_process$1 = require('mz/child_process');
var libRc = require('@pagedip/lib-rc');
var path = require('path');

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve$$1, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve$$1,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();



var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve$$1, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve$$1(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};











var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



































var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

function formatPackageName(pkg) {
  if (typeof pkg !== "string" || !pkg) {
    throw new Error("Expecting non-empty string for package name.");
  }

  if (!resolve.isPath.test(pkg) && pkg.substr(0, 9) !== "@pagedip/") {
    pkg = "@pagedip/" + pkg;
  }

  return pkg;
}

var npmCli = void 0;
var npmPath = function () {
  var _ref = asyncToGenerator(function* () {
    if (npmCli) return npmCli;
    var p = yield getNpmCliPath();
    npmCli = p;
    return p;
  });

  return function npmPath() {
    return _ref.apply(this, arguments);
  };
}();

var npmCommand = function () {
  var _ref2 = asyncToGenerator(function* (args) {
    var cli = yield npmPath();

    return yield new Promise(function (resolve$$1, reject) {
      var cp = child_process.spawn("node", [cli].concat(toConsumableArray(args)), {
        stdio: "inherit"
      });

      cp.on("exit", function (code) {
        code ? reject(new Error("npm command failed")) : resolve$$1();
      });

      cp.on("error", reject);
    });
  });

  return function npmCommand(_x) {
    return _ref2.apply(this, arguments);
  };
}();

function npmInstall(pkgname) {
  return npmCommand(["i", pkgname, "-g"]);
}

var install = (function () {
  var _ref = asyncToGenerator(function* (pkgs, opts) {
    if (!Array.isArray(pkgs)) pkgs = pkgs == null ? [] : [pkgs];

    opts = opts || {};

    for (var i = 0; i < pkgs.length; i++) {
      if (opts.logger) opts.logger.debug("Installing package '%s'", pkgs[i]);
      yield npmInstall(formatPackageName(pkgs[i]));
    }
  });

  function install(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return install;
})();

var load = (function () {
  var _ref = asyncToGenerator(function* (pkgname, opts) {
    var cliFile = yield resolve__default(pkgname, _extends({}, opts, {
      leads: ["@pagedip/", ""],
      packageFilter: function packageFilter(pkg) {
        if (pkg.command) pkg.main = pkg.command;else delete pkg.main;
        return pkg;
      }
    }));

    if (!cliFile) {
      var err = new Error("Could not find package '" + pkgname + "'.");
      err.code = "MODULE_NOT_FOUND";
      throw err;
    }

    return require(cliFile);
  });

  function load(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return load;
})();

var register = (function () {
  var _ref = asyncToGenerator(function* (license) {
    yield fs.writeFile(path.join(libRc.globalPath, ".pdlicense"), license);
    yield child_process$1.exec("npm config set @pagedip:registry https://license.system.pagedip.com/npm");
    yield child_process$1.exec("npm config set //license.system.pagedip.com/:_authToken " + license);
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

exports.install = install;
exports.load = load;
exports.register = register;
